@@ -405,6 +405,7 @@ struct drumkv1_gen
 {
 	drumkv1_port sample;
 	drumkv1_port reverse;
+	drumkv1_port offset;
 	drumkv1_port group;
 	drumkv1_port coarse;
 	drumkv1_port fine;
@@ -452,9 +453,9 @@ struct drumkv1_lfo
 
 struct drumkv1_dca
 {
-	drumkv1_port2 volume;
+	drumkv1_port volume;
 
-	drumkv1_env   env;
+	drumkv1_env  env;
 };
 
 
@@ -476,10 +477,10 @@ struct drumkv1_def
 
 struct drumkv1_out
 {
-	drumkv1_port2 width;
-	drumkv1_port2 panning;
-	drumkv1_port2 fxsend;
-	drumkv1_port2 volume;
+	drumkv1_port width;
+	drumkv1_port panning;
+	drumkv1_port fxsend;
+	drumkv1_port volume;
 };
 
 
@@ -487,11 +488,11 @@ struct drumkv1_out
 
 struct drumkv1_cho
 {
-	drumkv1_port2 wet;
-	drumkv1_port2 delay;
-	drumkv1_port2 feedb;
-	drumkv1_port2 rate;
-	drumkv1_port2 mod;
+	drumkv1_port wet;
+	drumkv1_port delay;
+	drumkv1_port feedb;
+	drumkv1_port rate;
+	drumkv1_port mod;
 };
 
 
@@ -499,10 +500,10 @@ struct drumkv1_cho
 
 struct drumkv1_fla
 {
-	drumkv1_port2 wet;
-	drumkv1_port2 delay;
-	drumkv1_port2 feedb;
-	drumkv1_port2 daft;
+	drumkv1_port wet;
+	drumkv1_port delay;
+	drumkv1_port feedb;
+	drumkv1_port daft;
 };
 
 
@@ -510,11 +511,11 @@ struct drumkv1_fla
 
 struct drumkv1_pha
 {
-	drumkv1_port2 wet;
-	drumkv1_port2 rate;
-	drumkv1_port2 feedb;
-	drumkv1_port2 depth;
-	drumkv1_port2 daft;
+	drumkv1_port wet;
+	drumkv1_port rate;
+	drumkv1_port feedb;
+	drumkv1_port depth;
+	drumkv1_port daft;
 };
 
 
@@ -522,10 +523,10 @@ struct drumkv1_pha
 
 struct drumkv1_del
 {
-	drumkv1_port2 wet;
-	drumkv1_port2 delay;
-	drumkv1_port2 feedb;
-	drumkv1_port2 bpm;
+	drumkv1_port wet;
+	drumkv1_port delay;
+	drumkv1_port feedb;
+	drumkv1_port bpm;
 };
 
 
@@ -533,11 +534,11 @@ struct drumkv1_del
 
 struct drumkv1_rev
 {
-	drumkv1_port2 wet;
-	drumkv1_port2 room;
-	drumkv1_port2 damp;
-	drumkv1_port2 feedb;
-	drumkv1_port2 width;
+	drumkv1_port wet;
+	drumkv1_port room;
+	drumkv1_port damp;
+	drumkv1_port feedb;
+	drumkv1_port width;
 };
 
 // dynamic(compressor/limiter)
@@ -696,8 +697,11 @@ void drumkv1_elem::updateEnvTimes ( float srate )
 	const float srate_ms = 0.001f * srate;
 
 	float envtime_msecs = 10000.0f * gen1.envtime0;
-	if (envtime_msecs < MIN_ENV_MSECS)
-		envtime_msecs = (gen1_sample.length() >> 1) / srate_ms;
+	if (envtime_msecs < MIN_ENV_MSECS) {
+		const uint32_t envtime_frames
+			= (gen1_sample.offsetEnd() - gen1_sample.offsetStart()) >> 1;
+		envtime_msecs = envtime_frames / srate_ms;
+	}
 	if (envtime_msecs < MIN_ENV_MSECS)
 		envtime_msecs = MIN_ENV_MSECS * 4.0f;
 
@@ -837,6 +841,13 @@ public:
 	void setReverse(bool bReverse);
 	bool isReverse() const;
 
+	void setOffset(bool bOffset);
+	bool isOffset() const;
+
+	void setOffsetRange(uint32_t iOffsetStart, uint32_t iOffsetEnd);
+	uint32_t offsetStart() const;
+	uint32_t offsetEnd() const;
+
 	void setTempo(float bpm);
 	float tempo() const;
 
@@ -857,6 +868,8 @@ public:
 	void resetParamValues(bool bSwap);
 	void reset();
 
+	bool sampleOffsetTest();
+
 	void midiInEnabled(bool on);
 	uint32_t midiInCount();
 
@@ -1315,6 +1328,33 @@ bool drumkv1_impl::isReverse (void) const
 }
 
 
+void drumkv1_impl::setOffset ( bool bOffset )
+{
+	if (m_elem) m_elem->element.setOffset(bOffset);
+}
+
+bool drumkv1_impl::isOffset (void) const
+{
+	return (m_elem ? m_elem->element.isOffset() : false);
+}
+
+
+void drumkv1_impl::setOffsetRange ( uint32_t iOffsetStart, uint32_t iOffsetEnd )
+{
+	if (m_elem) m_elem->element.setOffsetRange(iOffsetStart, iOffsetEnd);
+}
+
+uint32_t drumkv1_impl::offsetStart (void) const
+{
+	return (m_elem ? m_elem->element.offsetStart() : 0);
+}
+
+uint32_t drumkv1_impl::offsetEnd (void) const
+{
+	return (m_elem ? m_elem->element.offsetEnd() : 0);
+}
+
+
 void drumkv1_impl::setParamPort ( drumkv1::ParamIndex index, float *pfParam )
 {
 	static float s_fDummy = 0.0f;
@@ -1462,12 +1502,16 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 		if (++i >= size)
 			break;
 
-		// channel filter
-		if (!on)
-			continue;
-
+		// channel value
 		const int value = (data[i] & 0x7f);
 
+		// channel/controller filter
+		if (!on) {
+			if (status == 0xb0)
+				m_controls.process_enqueue(channel, key, value);
+			continue;
+		}
+
 		// note on
 		if (status == 0x90 && value > 0) {
 			drumkv1_voice *pv = m_notes[key];
@@ -2067,10 +2111,6 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 	elem = m_elem_list.next();
 	while (elem) {
-		elem->dca1.volume.tick(nframes);
-		elem->out1.width.tick(nframes);
-		elem->out1.panning.tick(nframes);
-		elem->out1.volume.tick(nframes);
 		elem->wid1.process(nframes);
 		elem->pan1.process(nframes);
 		elem->vol1.process(nframes);
@@ -2081,6 +2121,12 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 }
 
 
+bool drumkv1_impl::sampleOffsetTest (void)
+{
+	return (m_elem ? m_elem->element.sampleOffsetTest() : false);
+}
+
+
 //-------------------------------------------------------------------------
 // drumkv1 - decl.
 //
@@ -2214,6 +2260,35 @@ bool drumkv1::isReverse (void) const
 }
 
 
+void drumkv1::setOffset ( bool bOffset )
+{
+	m_pImpl->setOffset(bOffset);
+}
+
+bool drumkv1::isOffset (void) const
+{
+	return m_pImpl->isOffset();
+}
+
+
+void drumkv1::setOffsetRange ( uint32_t iOffsetStart, uint32_t iOffsetEnd )
+{
+	m_pImpl->setOffsetRange(iOffsetStart, iOffsetEnd);
+}
+
+
+uint32_t drumkv1::offsetStart (void) const
+{
+	return m_pImpl->offsetStart();
+}
+
+
+uint32_t drumkv1::offsetEnd (void) const
+{
+	return m_pImpl->offsetEnd();
+}
+
+
 void drumkv1::setTempo ( float bpm )
 {
 	m_pImpl->setTempo(bpm);
@@ -2299,6 +2374,12 @@ void drumkv1::reset (void)
 }
 
 
+bool drumkv1::sampleOffsetTest (void) const
+{
+	return m_pImpl->sampleOffsetTest();
+}
+
+
 //-------------------------------------------------------------------------
 // drumkv1_element - decl.
 //
@@ -2351,6 +2432,36 @@ bool drumkv1_element::isReverse (void) const
 }
 
 
+void drumkv1_element::setOffset ( bool bOffset )
+{
+	if (m_pElem) m_pElem->gen1_sample.setOffset(bOffset);
+}
+
+bool drumkv1_element::isOffset (void) const
+{
+	return (m_pElem ? m_pElem->gen1_sample.isOffset() : false);
+}
+
+
+void drumkv1_element::setOffsetRange ( uint32_t iOffsetStart, uint32_t iOffsetEnd )
+{
+	if (m_pElem) {
+		m_pElem->gen1_sample.setOffsetRange(iOffsetStart, iOffsetEnd);
+		m_pElem->updateEnvTimes(m_pElem->gen1_sample.sampleRate());
+	}
+}
+
+uint32_t drumkv1_element::offsetStart (void) const
+{
+	return (m_pElem ? m_pElem->gen1_sample.offsetStart() : 0);
+}
+
+uint32_t drumkv1_element::offsetEnd (void) const
+{
+	return (m_pElem ? m_pElem->gen1_sample.offsetEnd() : 0);
+}
+
+
 void drumkv1_element::setParamPort ( drumkv1::ParamIndex index, float *pfParam )
 {
 	drumkv1_port *pParamPort = paramPort(index);
@@ -2369,6 +2480,7 @@ drumkv1_port *drumkv1_element::paramPort ( drumkv1::ParamIndex index )
 	switch (index) {
 //	case drumkv1::GEN1_SAMPLE:   pParamPort = &m_pElem->gen1.sample;     break;
 	case drumkv1::GEN1_REVERSE:  pParamPort = &m_pElem->gen1.reverse;    break;
+	case drumkv1::GEN1_OFFSET:   pParamPort = &m_pElem->gen1.offset;     break;
 	case drumkv1::GEN1_GROUP:    pParamPort = &m_pElem->gen1.group;      break;
 	case drumkv1::GEN1_COARSE:   pParamPort = &m_pElem->gen1.coarse;     break;
 	case drumkv1::GEN1_FINE:     pParamPort = &m_pElem->gen1.fine;       break;
@@ -2447,6 +2559,15 @@ void drumkv1_element::resetParamValues ( bool bSwap )
 }
 
 
+bool drumkv1_element::sampleOffsetTest (void)
+{
+	if (m_pElem)
+		return m_pElem->gen1_sample.offset_test(*m_pElem->gen1.offset > 0.5f);
+	else
+		return false;
+}
+
+
 // MIDI input asynchronous status notification accessors
 
 void drumkv1::midiInEnabled ( bool on )
