@@ -77,6 +77,13 @@ public:
 	void setReverse(bool bReverse);
 	bool isReverse() const;
 
+	void setOffset(bool bOffset);
+	bool isOffset() const;
+
+	void setOffsetRange(uint32_t iOffsetStart, uint32_t iOffsetEnd);
+	uint32_t offsetStart() const;
+	uint32_t offsetEnd() const;
+
 	void setTempo(float bpm);
 	float tempo() const;
 
@@ -84,6 +91,7 @@ public:
 
 		GEN1_SAMPLE = 0,
 		GEN1_REVERSE,
+		GEN1_OFFSET,
 		GEN1_GROUP,
 		GEN1_COARSE,
 		GEN1_FINE,
@@ -175,6 +183,8 @@ public:
 	void process_midi(uint8_t *data, uint32_t size);
 	void process(float **ins, float **outs, uint32_t nframes);
 
+	bool sampleOffsetTest() const;
+
 	virtual void updatePreset(bool bDirty) = 0;
 
 	void midiInEnabled(bool on);
@@ -215,6 +225,13 @@ public:
 	void setReverse(bool bReverse);
 	bool isReverse() const;
 
+	void setOffset(bool bOffset);
+	bool isOffset() const;
+
+	void setOffsetRange(uint32_t iOffsetStart, uint32_t iOffsetEnd);
+	uint32_t offsetStart() const;
+	uint32_t offsetEnd() const;
+
 	void setParamPort(drumkv1::ParamIndex index, float *pfParam);
 	drumkv1_port *paramPort(drumkv1::ParamIndex index);
 
@@ -223,6 +240,8 @@ public:
 
 	void resetParamValues(bool bSwap);
 
+	bool sampleOffsetTest();
+
 private:
 
 	drumkv1_elem *m_pElem;
