@@ -50,6 +50,7 @@ struct ParamInfo {
 	// name            type,           def,    min,    max
 	{ "GEN1_SAMPLE",   PARAM_INT,    36.0f,   0.0f, 127.0f }, // GEN1 Sample
 	{ "GEN1_REVERSE",  PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // GEN1 Reverse
+	{ "GEN1_OFFSET",   PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // GEN1 Offset
 	{ "GEN1_GROUP",    PARAM_FLOAT,   0.0f,   0.0f, 128.0f }, // GEN1 Group
 	{ "GEN1_COARSE",   PARAM_FLOAT,   0.0f,  -4.0f,   4.0f }, // GEN1 Coarse
 	{ "GEN1_FINE",     PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // GEN1 Fine
@@ -222,12 +223,17 @@ void drumkv1_param::loadElements (
 				if (eChild.isNull())
 					continue;
 				if (eChild.tagName() == "sample") {
-				//	int index = eChild.attribute("index").toInt();
+				//	const int index = eChild.attribute("index").toInt();
+					const uint32_t iOffsetStart
+						= eChild.attribute("offset-start").toULong();
+					const uint32_t iOffsetEnd
+						= eChild.attribute("offset-end").toULong();
 					QFileInfo fi(eChild.text());
 					if (fi.isSymLink())
 						fi.setFile(fi.symLinkTarget());
 					element->setSampleFile(
 						mapPath.absolutePath(fi.filePath()).toUtf8().constData());
+					element->setOffsetRange(iOffsetStart, iOffsetEnd);
 				}
 				else
 				if (eChild.tagName() == "params") {
@@ -289,6 +295,8 @@ void drumkv1_param::saveElements (
 		QDomElement eSample = doc.createElement("sample");
 		eSample.setAttribute("index", 0);
 		eSample.setAttribute("name", "GEN1_SAMPLE");
+		eSample.setAttribute("offset-start", element->offsetStart());
+		eSample.setAttribute("offset-end", element->offsetEnd());
 		eSample.appendChild(doc.createTextNode(
 			mapPath.abstractPath(fi.absoluteFilePath())));
 		eElement.appendChild(eSample);
