@@ -1,7 +1,7 @@
 // drumkv1_sample.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -60,7 +60,9 @@ private:
 drumkv1_sample::drumkv1_sample ( drumkv1 *pDrumk, float srate )
 	: m_srate(srate), m_filename(NULL), m_nchannels(0),
 		m_rate0(0.0f), m_freq0(1.0f), m_ratio(0.0f),
-		m_nframes(0), m_pframes(NULL), m_reverse(false)
+		m_nframes(0), m_pframes(NULL), m_reverse(false),
+		m_offset(false), m_offset_start(0), m_offset_end(0),
+		m_offset_phase0(0.0f), m_offset_end2(0)
 {
 	m_reverse_sched = new drumkv1_reverse_sched(pDrumk, this);
 }
@@ -148,6 +150,7 @@ bool drumkv1_sample::open ( const char *filename, float freq0 )
 
 	reset(freq0);
 
+	setOffset(m_offset);
 	return true;
 }
 
@@ -167,6 +170,8 @@ void drumkv1_sample::close (void)
 	m_rate0     = 0.0f;
 	m_nchannels = 0;
 
+	setOffsetRange(0, 0);
+
 	if (m_filename) {
 		::free(m_filename);
 		m_filename = NULL;
@@ -200,4 +205,60 @@ void drumkv1_sample::reverse_sync (void)
 }
 
 
+// offset range.
+void drumkv1_sample::setOffsetRange ( uint32_t start, uint32_t end )
+{
+	if (start > m_nframes)
+		start = m_nframes;
+
+	if (end > m_nframes || start >= end)
+		end = m_nframes;
+
+	if (start < end) {
+		m_offset_start = start;
+		m_offset_end = end;
+		m_offset_phase0 = float(zero_crossing(start, NULL));
+		m_offset_end2 = zero_crossing(end, NULL);
+	} else {
+		m_offset_start = 0;
+		m_offset_end = m_nframes;
+		m_offset_phase0 = 0.0f;
+		m_offset_end2 = m_nframes;
+	}
+}
+
+
+// zero-crossing aliasing (single channel).
+uint32_t drumkv1_sample::zero_crossing_k (
+	uint32_t i, uint16_t k, int *slope ) const
+{
+	const float *frames = m_pframes[k];
+	const int s0 = (slope ? *slope : 0);
+
+	if (i > 0) --i;
+	float v0 = frames[i];
+	for (++i; i < m_nframes; ++i) {
+		const float v1 = frames[i];
+		if ((0 >= s0 && v0 >= 0.0f && 0.0f >= v1) ||
+			(s0 >= 0 && v1 >= 0.0f && 0.0f >= v0)) {
+			if (slope && s0 == 0) *slope = (v1 < v0 ? -1 : +1);
+			return i;
+		}
+		v0 = v1;
+	}
+
+	return m_nframes;
+}
+
+
+// zero-crossing aliasing (median).
+uint32_t drumkv1_sample::zero_crossing ( uint32_t i, int *slope ) const
+{
+	uint32_t sum = 0;
+	for (uint16_t k = 0; k < m_nchannels; ++k)
+		sum += zero_crossing_k(i, k, slope);
+	return (sum / m_nchannels);
+}
+
+
 // end of drumkv1_sample.cpp
