@@ -1,7 +1,7 @@
 // drumkv1_sample.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -73,6 +73,45 @@ public:
 	void reverse_sched();
 	void reverse_sync();
 
+	// offset mode.
+	void setOffset(bool offset)
+	{
+		m_offset = offset;
+
+		if (m_offset_start >= m_offset_end) {
+			m_offset_start = 0;
+			m_offset_end = m_nframes;
+			m_offset_phase0 = 0.0f;
+		}
+
+		m_offset_end2 = (m_offset ? m_offset_end : m_nframes);
+	}
+
+	bool isOffset() const
+		{ return m_offset && (m_offset_start < m_offset_end); }
+
+	// offset change.
+	bool offset_test(bool offset)
+	{
+		if ((m_offset && !offset) || (!m_offset && offset)) {
+			setOffset(offset);
+			return true;
+		} else {
+			return false;
+		}
+	}
+
+	// sample start/end points (offsets)
+	void setOffsetRange(uint32_t start, uint32_t end);
+
+	uint32_t offsetStart() const
+		{ return m_offset_start; }
+	uint32_t offsetEnd() const
+		{ return m_offset_end; }
+
+	float offsetPhase0() const
+		{ return (m_offset ? m_offset_phase0 : 0.0f); }
+
 	// init.
 	bool open(const char *filename, float freq0 = 1.0f);
 	void close();
@@ -105,8 +144,14 @@ public:
 		{ return m_pframes[k]; }
 
 	// predicate.
-	bool isOver(uint32_t frame) const
-		{ return !m_pframes || (frame >= m_nframes); }
+	bool isOver(uint32_t index) const
+		{ return !m_pframes || (index >= m_offset_end2); }
+
+protected:
+
+	// zero-crossing aliasing .
+	uint32_t zero_crossing_k(uint32_t i, uint16_t k, int *slope) const;
+	uint32_t zero_crossing(uint32_t i, int *slope) const;
 
 private:
 
@@ -121,6 +166,12 @@ private:
 	float  **m_pframes;
 	bool     m_reverse;
 
+	bool     m_offset;
+	uint32_t m_offset_start;
+	uint32_t m_offset_end;
+	float    m_offset_phase0;
+	uint32_t m_offset_end2;
+
 	drumkv1_reverse_sched *m_reverse_sched;
 };
 
@@ -150,10 +201,9 @@ public:
 	// begin.
 	void start(void)
 	{
-		m_phase = 0.0f;
+		m_phase = (m_sample ? m_sample->offsetPhase0() : 0.0f);
 		m_index = 0;
 		m_alpha = 0.0f;
-		m_frame = 0;
 	}
 
 	// iterate.
@@ -164,9 +214,6 @@ public:
 		m_index  = int(m_phase);
 		m_alpha  = m_phase - float(m_index);
 		m_phase += delta;
-
-		if (m_frame < m_index)
-			m_frame = m_index;
 	}
 
 	// sample.
@@ -193,7 +240,7 @@ public:
 
 	// predicate.
 	bool isOver() const
-		{ return m_sample->isOver(m_frame); }
+		{ return m_sample->isOver(m_index); }
 
 private:
 
@@ -203,7 +250,6 @@ private:
 	float    m_phase;
 	uint32_t m_index;
 	float    m_alpha;
-	uint32_t m_frame;
 };
 
 
