@@ -86,6 +86,22 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	for (int iTab = 0; iTab < m_ui.StackedWidget->count(); ++iTab)
 		m_ui.TabBar->addTab(m_ui.StackedWidget->widget(iTab)->windowTitle());
 
+	// Offset/Loop range font.
+	const QFont& font = m_ui.Gen1ReverseKnob->font();
+	m_ui.Gen1OffsetRangeLabel->setFont(font);
+	m_ui.Gen1OffsetStartSpinBox->setFont(font);
+	m_ui.Gen1OffsetEndSpinBox->setFont(font);
+
+	const QFontMetrics fm(font);
+	m_ui.Gen1OffsetStartSpinBox->setMaximumHeight(fm.height() + 6);
+	m_ui.Gen1OffsetEndSpinBox->setMaximumHeight(fm.height() + 6);
+
+	m_ui.Gen1OffsetStartSpinBox->setAccelerated(true);
+	m_ui.Gen1OffsetEndSpinBox->setAccelerated(true);
+
+	m_ui.Gen1OffsetStartSpinBox->setMinimum(0);
+	m_ui.Gen1OffsetEndSpinBox->setMinimum(0);
+
 	// Swappable params A/B group.
 	QButtonGroup *pSwapParamsGroup = new QButtonGroup(this);
 	pSwapParamsGroup->addButton(m_ui.SwapParamsAButton);
@@ -144,6 +160,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	states << tr("On");
 #if 0
 	m_ui.Gen1ReverseKnob->insertItems(0, states);
+	m_ui.Gen1OffsetKnob->insertItems(0, states);
 
 	m_ui.Lfo1SyncKnob->insertItems(0, states);
 
@@ -222,6 +239,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	// GEN1
 	setParamKnob(drumkv1::GEN1_REVERSE, m_ui.Gen1ReverseKnob);
+	setParamKnob(drumkv1::GEN1_OFFSET,  m_ui.Gen1OffsetKnob);
 	setParamKnob(drumkv1::GEN1_GROUP,   m_ui.Gen1GroupKnob);
 	setParamKnob(drumkv1::GEN1_COARSE,  m_ui.Gen1CoarseKnob);
 	setParamKnob(drumkv1::GEN1_FINE,    m_ui.Gen1FineKnob);
@@ -463,9 +481,11 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(resetPresetFile()),
 		SLOT(resetParams()));
 
-	// Common context menu...
+	// Common context menu policies...
 	m_ui.Elements->setContextMenuPolicy(Qt::CustomContextMenu);
 	m_ui.Gen1Sample->setContextMenuPolicy(Qt::CustomContextMenu);
+	m_ui.Gen1OffsetStartSpinBox->setContextMenuPolicy(Qt::CustomContextMenu);
+	m_ui.Gen1OffsetEndSpinBox->setContextMenuPolicy(Qt::CustomContextMenu);
 
 	QObject::connect(m_ui.Elements,
 		SIGNAL(customContextMenuRequested(const QPoint&)),
@@ -474,6 +494,23 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(customContextMenuRequested(const QPoint&)),
 		SLOT(contextMenuRequest(const QPoint&)));
 
+	QObject::connect(m_ui.Gen1Sample,
+		SIGNAL(offsetRangeChanged()),
+		SLOT(offsetRangeChanged()));
+
+	QObject::connect(m_ui.Gen1OffsetStartSpinBox,
+		SIGNAL(valueChanged(uint32_t)),
+		SLOT(offsetStartChanged()));
+	QObject::connect(m_ui.Gen1OffsetEndSpinBox,
+		SIGNAL(valueChanged(uint32_t)),
+		SLOT(offsetEndChanged()));
+
+	QObject::connect(m_ui.Gen1OffsetStartSpinBox,
+		SIGNAL(customContextMenuRequested(const QPoint&)),
+		SLOT(spinboxContextMenu(const QPoint&)));
+	QObject::connect(m_ui.Gen1OffsetEndSpinBox,
+		SIGNAL(customContextMenuRequested(const QPoint&)),
+		SLOT(spinboxContextMenu(const QPoint&)));
 
 	// Swap params A/B
 	QObject::connect(m_ui.SwapParamsAButton,
@@ -498,13 +535,17 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(triggered(bool)),
 		SLOT(helpAboutQt()));
 
-	// General knob/dial  behavior init...
+	// General knob/dial behavior init...
 	drumkv1_config *pConfig = drumkv1_config::getInstance();
 	if (pConfig) {
 		drumkv1widget_dial::setDialMode(
 			drumkv1widget_dial::DialMode(pConfig->iKnobDialMode));
 		drumkv1widget_edit::setEditMode(
 			drumkv1widget_edit::EditMode(pConfig->iKnobEditMode));
+		const drumkv1widget_spinbox::Format format
+			= drumkv1widget_spinbox::Format(pConfig->iFrameTimeFormat);
+		m_ui.Gen1OffsetStartSpinBox->setFormat(format);
+		m_ui.Gen1OffsetEndSpinBox->setFormat(format);
 	}
 
 	// Epilog.
@@ -664,20 +705,27 @@ void drumkv1widget::paramChanged ( float fValue )
 // Update local tied widgets.
 void drumkv1widget::updateParamEx ( drumkv1::ParamIndex index, float fValue )
 {
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi == NULL)
+		return;
+
 	++m_iUpdate;
 
 	switch (index) {
-#if 0//--updateSchedNotify(drumkv1_sched::Sample, 0);
+#if 1//--updateSchedNotify(drumkv1_sched::Sample, 0);
 	case drumkv1::GEN1_REVERSE: {
-		drumkv1_ui *pDrumkUi = ui_instance();
-		if (pDrumkUi) {
-			const bool bReverse = bool(fValue > 0.0f);
-			pDrumkUi->setReverse(bReverse);
-			updateSample(pDrumkUi->sample());
-		}
+		const bool bReverse = bool(fValue > 0.0f);
+		pDrumkUi->setReverse(bReverse);
+		updateSample(pDrumkUi->sample());
 		break;
 	}
 #endif
+	case drumkv1::GEN1_OFFSET: {
+		const bool bOffset = bool(fValue > 0.0f);
+		pDrumkUi->setOffset(bOffset);
+		updateOffset(pDrumkUi->sample());
+		break;
+	}
 	case drumkv1::DCF1_SLOPE:
 		m_ui.Dcf1TypeKnob->setEnabled(int(fValue) != 3); // !Formant
 		// Fall thru...
@@ -1027,6 +1075,22 @@ void drumkv1widget::updateSample ( drumkv1_sample *pSample, bool bDirty )
 	m_ui.Gen1Sample->setSampleName(currentNoteName());
 	m_ui.Gen1Sample->setSample(pSample);
 
+	++m_iUpdate;
+	if (pSample) {
+		m_ui.Gen1Sample->setOffset(pSample->isOffset());
+		m_ui.Gen1Sample->setOffsetStart(pSample->offsetStart());
+		m_ui.Gen1Sample->setOffsetEnd(pSample->offsetEnd());
+		activateParamKnobs(pSample->filename() != NULL);
+		updateOffset(pSample);
+	} else {
+		m_ui.Gen1Sample->setOffset(false);
+		m_ui.Gen1Sample->setOffsetStart(0);
+		m_ui.Gen1Sample->setOffsetEnd(0);
+		activateParamKnobs(false);
+		updateOffset(NULL);
+	}
+	--m_iUpdate;
+
 	if (pSample && bDirty)
 		updateDirtyPreset(true);
 }
@@ -1257,7 +1321,7 @@ void drumkv1widget::updateElement (void)
 
 	drumkv1_element *element = pDrumkUi->element(iCurrentNote);
 	if (element) {
-		activateParamKnobs(true);
+	//	activateParamKnobs(true);
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 			const float fValue = element->paramValue(index);
@@ -1267,6 +1331,7 @@ void drumkv1widget::updateElement (void)
 				pParam->setValue(fValue);
 			}
 			updateParam(index, fValue);
+			updateParamEx(index, fValue);
 			m_params_ab[i] = fValue;
 		}
 		updateSample(pDrumkUi->sample());
@@ -1274,7 +1339,7 @@ void drumkv1widget::updateElement (void)
 	} else {
 		updateSample(NULL);
 		resetParamValues(drumkv1::NUM_ELEMENT_PARAMS);
-		activateParamKnobs(false);
+	//	activateParamKnobs(false);
 	}
 
 	--m_iUpdate;
@@ -1305,6 +1370,110 @@ void drumkv1widget::resetElement (void)
 }
 
 
+// Offset start change.
+void drumkv1widget::offsetStartChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi) {
+		const uint32_t iOffsetStart = m_ui.Gen1OffsetStartSpinBox->value();
+		const uint32_t iOffsetEnd = pDrumkUi->offsetEnd();
+		pDrumkUi->setOffsetRange(iOffsetStart, iOffsetEnd);
+		updateOffset(pDrumkUi->sample(), true);
+	}
+	--m_iUpdate;
+}
+
+
+void drumkv1widget::offsetEndChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi) {
+		const uint32_t iOffsetStart = pDrumkUi->offsetStart();
+		const uint32_t iOffsetEnd = m_ui.Gen1OffsetEndSpinBox->value();
+		pDrumkUi->setOffsetRange(iOffsetStart, iOffsetEnd);
+		updateOffset(pDrumkUi->sample(), true);
+	}
+	--m_iUpdate;
+}
+
+
+// Offset points changed (from UI).
+void drumkv1widget::offsetRangeChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi) {
+		const uint32_t iOffsetStart = m_ui.Gen1Sample->offsetStart();
+		const uint32_t iOffsetEnd   = m_ui.Gen1Sample->offsetEnd();
+		pDrumkUi->setOffsetRange(iOffsetStart, iOffsetEnd);
+		updateOffset(pDrumkUi->sample(), true);
+	}
+	--m_iUpdate;
+}
+
+
+// Update offset change status.
+void drumkv1widget::updateOffset ( drumkv1_sample *pSample, bool bDirty )
+{
+	if (pSample) {
+		const bool     bOffset    = pSample->isOffset();
+		const uint32_t iOffsetStart = pSample->offsetStart();
+		const uint32_t iOffsetEnd = pSample->offsetEnd();
+		const uint32_t nframes    = pSample->length();
+		const float srate = pSample->sampleRate();
+		m_ui.Gen1OffsetRangeLabel->setEnabled(bOffset);
+		m_ui.Gen1OffsetStartSpinBox->setSampleRate(srate);
+		m_ui.Gen1OffsetStartSpinBox->setEnabled(bOffset);
+		m_ui.Gen1OffsetStartSpinBox->setMinimum(0);
+		m_ui.Gen1OffsetStartSpinBox->setMaximum(iOffsetEnd);
+		m_ui.Gen1OffsetStartSpinBox->setValue(iOffsetStart);
+		m_ui.Gen1OffsetEndSpinBox->setSampleRate(srate);
+		m_ui.Gen1OffsetEndSpinBox->setEnabled(bOffset);
+		m_ui.Gen1OffsetEndSpinBox->setMinimum(iOffsetStart);
+		m_ui.Gen1OffsetEndSpinBox->setMaximum(nframes);
+		m_ui.Gen1OffsetEndSpinBox->setValue(iOffsetEnd);
+		m_ui.Gen1Sample->setOffsetStart(iOffsetStart);
+		m_ui.Gen1Sample->setOffsetEnd(iOffsetEnd);
+		m_ui.Gen1Sample->setOffset(bOffset);
+		if (bDirty) {
+			QString sMessage;
+			if (bOffset) {
+				sMessage.append(tr("Offset: %1 - %2")
+					.arg(m_ui.Gen1Sample->textFromValue(iOffsetStart))
+					.arg(m_ui.Gen1Sample->textFromValue(iOffsetEnd)));
+			}
+			if (!sMessage.isEmpty())
+				m_ui.StatusBar->showMessage(sMessage, 3000);
+			updateDirtyPreset(true);
+		}
+	} else {
+		m_ui.Gen1OffsetRangeLabel->setEnabled(false);
+		m_ui.Gen1OffsetStartSpinBox->setEnabled(false);
+		m_ui.Gen1OffsetStartSpinBox->setMinimum(0);
+		m_ui.Gen1OffsetStartSpinBox->setMaximum(0);
+		m_ui.Gen1OffsetStartSpinBox->setValue(0);
+		m_ui.Gen1OffsetEndSpinBox->setEnabled(false);
+		m_ui.Gen1OffsetEndSpinBox->setMinimum(0);
+		m_ui.Gen1OffsetEndSpinBox->setMaximum(0);
+		m_ui.Gen1OffsetEndSpinBox->setValue(0);
+		m_ui.Gen1Sample->setOffsetStart(0);
+		m_ui.Gen1Sample->setOffsetEnd(0);
+		m_ui.Gen1Sample->setOffset(false);
+	}
+}
+
+
 // (En|Dis)able all param/knobs.
 void drumkv1widget::activateParamKnobs ( bool bEnabled )
 {
@@ -1559,4 +1728,43 @@ void drumkv1widget::paramContextMenu ( const QPoint& pos )
 }
 
 
+// Format changes (spinbox).
+void drumkv1widget::spinboxContextMenu ( const QPoint& pos )
+{
+	drumkv1widget_spinbox *pSpinBox
+		= qobject_cast<drumkv1widget_spinbox *> (sender());
+	if (pSpinBox == NULL)
+		return;
+
+	drumkv1widget_spinbox::Format format = pSpinBox->format();
+
+	QMenu menu(this);
+	QAction *pAction;
+
+	pAction = menu.addAction(tr("&Frames"));
+	pAction->setCheckable(true);
+	pAction->setChecked(format == drumkv1widget_spinbox::Frames);
+	pAction->setData(int(drumkv1widget_spinbox::Frames));
+
+	pAction = menu.addAction(tr("&Time"));
+	pAction->setCheckable(true);
+	pAction->setChecked(format == drumkv1widget_spinbox::Time);
+	pAction->setData(int(drumkv1widget_spinbox::Time));
+
+	pAction = menu.exec(pSpinBox->mapToGlobal(pos));
+	if (pAction == NULL)
+		return;
+
+	format = drumkv1widget_spinbox::Format(pAction->data().toInt());
+	if (format != pSpinBox->format()) {
+		drumkv1_config *pConfig = drumkv1_config::getInstance();
+		if (pConfig) {
+			pConfig->iFrameTimeFormat = int(format);
+			m_ui.Gen1OffsetStartSpinBox->setFormat(format);
+			m_ui.Gen1OffsetEndSpinBox->setFormat(format);
+		}
+	}
+}
+
+
 // end of drumkv1widget.cpp
