@@ -134,6 +134,13 @@ protected slots:
 	// Swap params A/B.
 	void swapParams(bool bOn);
 
+	// Offset point changed.
+	void offsetStartChanged();
+	void offsetEndChanged();
+
+	// Offset points changed (from UI).
+	void offsetRangeChanged();
+
 	// Notification updater.
 	void updateSchedNotify(int stype, int sid);
 
@@ -143,6 +150,9 @@ protected slots:
 	// Param knob context menu.
 	void paramContextMenu(const QPoint& pos);
 
+	// Format changes (spinbox).
+	void spinboxContextMenu(const QPoint& pos);
+
 	// Menu actions.
 	void helpConfigure();
 
@@ -183,6 +193,9 @@ protected:
 	// Sample updater.
 	void updateSample(drumkv1_sample *pSample, bool bDirty = false);
 
+	// Update offset change status.
+	void updateOffset(drumkv1_sample *pSample, bool bDirty = false);
+
 	// Param port method.
 	virtual void updateParam(drumkv1::ParamIndex index, float fValue) const = 0;
 
